@@ -44,13 +44,19 @@ G_BEGIN_DECLS
 typedef enum
 {
   G_MODULE_BIND_LAZY	= 1 << 0,
-  G_MODULE_BIND_MASK	= 0x01
+  G_MODULE_BIND_LOCAL	= 1 << 1,
+  G_MODULE_BIND_MASK	= 0x03
 } GModuleFlags;
 
 typedef	struct _GModule			 GModule;
 typedef const gchar* (*GModuleCheckInit) (GModule	*module);
 typedef void	     (*GModuleUnload)	 (GModule	*module);
 
+#ifdef G_OS_WIN32
+#define g_module_open g_module_open_utf8
+#define g_module_name g_module_name_utf8
+#endif
+
 /* return TRUE if dynamic module loading is supported */
 gboolean	g_module_supported	   (void) G_GNUC_CONST;
 
@@ -67,12 +73,12 @@ void                  g_module_make_resident (GModule      *module);
 /* query the last module error as a string */
 G_CONST_RETURN gchar* g_module_error         (void);
 
-/* retrive a symbol pointer from `module', returns TRUE on success */
+/* retrieve a symbol pointer from `module', returns TRUE on success */
 gboolean              g_module_symbol        (GModule      *module,
 					      const gchar  *symbol_name,
 					      gpointer     *symbol);
 
-/* retrive the file name from an existing module */
+/* retrieve the file name from an existing module */
 G_CONST_RETURN gchar* g_module_name          (GModule      *module);
 
 /* Build the actual file name containing a module. `directory' is the
